@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "3.6.0"
+#define BARESIP_VERSION "3.15.0"
 
 
 #ifndef NET_MAX_NS
@@ -79,6 +79,12 @@ enum jbuf_type {
 	JBUF_ADAPTIVE
 };
 
+/** Defines the incoming out-of-dialog request mode */
+enum inreq_mode {
+	INREQ_MODE_OFF = 0,
+	INREQ_MODE_ON,
+};
+
 struct account;
 
 int account_alloc(struct account **accp, const char *sipaddr);
@@ -92,6 +98,7 @@ int account_set_sipnat(struct account *acc, const char *sipnat);
 int account_set_answermode(struct account *acc, enum answermode mode);
 int account_set_rel100_mode(struct account *acc, enum rel100_mode mode);
 int account_set_dtmfmode(struct account *acc, enum dtmfmode mode);
+int account_set_inreq_mode(struct account *acc, enum inreq_mode mode);
 int account_set_display_name(struct account *acc, const char *dname);
 int account_set_regint(struct account *acc, uint32_t regint);
 int account_set_stun_uri(struct account *acc, const char *uri);
@@ -99,6 +106,8 @@ int account_set_stun_host(struct account *acc, const char *host);
 int account_set_stun_port(struct account *acc, uint16_t port);
 int account_set_stun_user(struct account *acc, const char *user);
 int account_set_stun_pass(struct account *acc, const char *pass);
+int account_set_ausrc_dev(struct account *acc, const char *dev);
+int account_set_auplay_dev(struct account *acc, const char *dev);
 int account_set_mediaenc(struct account *acc, const char *mediaenc);
 int account_set_medianat(struct account *acc, const char *medianat);
 int account_set_audio_codecs(struct account *acc, const char *codecs);
@@ -106,6 +115,7 @@ int account_set_video_codecs(struct account *acc, const char *codecs);
 int account_set_mwi(struct account *acc, bool value);
 int account_set_call_transfer(struct account *acc, bool value);
 int account_set_rtcp_mux(struct account *acc, bool value);
+void account_set_catchall(struct account *acc, bool value);
 int account_auth(const struct account *acc, char **username, char **password,
 		 const char *realm);
 struct list *account_aucodecl(const struct account *acc);
@@ -120,6 +130,7 @@ uint32_t account_prio(const struct account *acc);
 enum answermode account_answermode(const struct account *acc);
 enum rel100_mode account_rel100_mode(const struct account *acc);
 enum dtmfmode account_dtmfmode(const struct account *acc);
+enum inreq_mode account_inreq_mode(const struct account *acc);
 const char *account_display_name(const struct account *acc);
 const char *account_aor(const struct account *acc);
 const char *account_auth_user(const struct account *acc);
@@ -153,6 +164,7 @@ void account_set_autelev_pt(struct account *acc, uint32_t pt);
 uint32_t account_autelev_pt(struct account *acc);
 const char* account_uas_user(const struct account *acc);
 const char* account_uas_pass(const struct account *acc);
+bool account_uas_isset(const struct account *acc);
 
 /*
  * Call
@@ -221,6 +233,7 @@ bool call_has_audio(const struct call *call);
 bool call_has_video(const struct call *call);
 bool call_early_video_available(const struct call *call);
 bool call_refresh_allowed(const struct call *call);
+bool call_ack_pending(const struct call *call);
 int  call_transfer(struct call *call, const char *uri);
 int  call_replace_transfer(struct call *target_call, struct call *source_call);
 int  call_status(struct re_printf *pf, const struct call *call);
@@ -254,16 +267,20 @@ struct call  *call_find_linenum(const struct list *calls, uint32_t linenum);
 struct call  *call_find_id(const struct list *calls, const char *id);
 void call_set_current(struct list *calls, struct call *call);
 const struct list *call_get_custom_hdrs(const struct call *call);
-int call_set_media_direction(struct call *call, enum sdp_dir a,
+void call_set_media_direction(struct call *call, enum sdp_dir a,
 			     enum sdp_dir v);
-int call_set_media_estdir(struct call *call, enum sdp_dir a, enum sdp_dir v);
+void call_set_mdir(struct call *call, enum sdp_dir a, enum sdp_dir v);
+void call_set_media_estdir(struct call *call, enum sdp_dir a, enum sdp_dir v);
 void call_start_answtmr(struct call *call, uint32_t ms);
 bool          call_supported(struct call *call, uint16_t tags);
 const char   *call_user_data(const struct call *call);
 int call_set_user_data(struct call *call, const char *user_data);
 void call_set_evstop(struct call *call, bool stop);
 bool call_is_evstop(struct call *call);
-bool call_sent_answer(const struct call *call);
+int call_msg_src(const struct call *call, struct sa *sa);
+enum sip_transp call_transp(const struct call *call);
+enum sdp_neg_state call_sdp_neg_state(const struct call *call);
+bool call_sdp_change_allowed(const struct call *call);
 
 /*
  * Custom headers
@@ -324,6 +341,15 @@ enum audio_mode {
 	AUDIO_MODE_THREAD,           /**< Use dedicated thread          */
 };
 
+/** RTP receive mode */
+enum rtp_receive_mode {
+	RECEIVE_MODE_MAIN = 0,  /**< RTP RX is processed in main thread      */
+	RECEIVE_MODE_THREAD,    /**< RTP RX is processed in separate thread  */
+};
+
+enum rtp_receive_mode resolve_receive_mode(const struct pl *fmt);
+const char *rtp_receive_mode_str(enum rtp_receive_mode rxmode);
+
 
 /** SIP User-Agent */
 struct config_sip {
@@ -335,7 +361,10 @@ struct config_sip {
 	uint32_t transports;    /**< Supported transports mask      */
 	enum sip_transp transp; /**< Default outgoing SIP transport protocol */
 	bool verify_server;     /**< Enable SIP TLS verify server   */
+	bool verify_client;     /**< Enable SIP TLS verify client   */
+	enum tls_resume_mode tls_resume; /** TLS resumption mode    */
 	uint8_t tos;            /**< Type-of-Service for SIP        */
+	uint32_t reg_filt;	/**< Registrar filter transport mask*/
 };
 
 /** Call config */
@@ -403,6 +432,7 @@ struct config_avt {
 	bool rtp_stats;         /**< Enable RTP statistics          */
 	uint32_t rtp_timeout;   /**< RTP Timeout in seconds (0=off) */
 	bool bundle;            /**< Media Multiplexing (BUNDLE)    */
+	enum rtp_receive_mode rxmode;   /**< RTP RX processing mode */
 };
 
 /** Network Configuration */
@@ -542,6 +572,7 @@ struct ausrc_prm {
 	uint8_t    ch;          /**< Number of channels         */
 	uint32_t   ptime;       /**< Wanted packet-time in [ms] */
 	int        fmt;         /**< Sample format (enum aufmt) */
+	size_t     duration;    /**< Duration in [ms], 0 for infinite        */
 };
 
 typedef void (ausrc_read_h)(struct auframe *af, void *arg);
@@ -645,6 +676,7 @@ typedef int (aufilt_decode_h)(struct aufilt_dec_st *st,
 struct aufilt {
 	struct le le;
 	const char *name;
+	bool enabled;
 	aufilt_encupd_h *encupdh;
 	aufilt_encode_h *ench;
 	aufilt_decupd_h *decupdh;
@@ -653,6 +685,7 @@ struct aufilt {
 
 void aufilt_register(struct list *aufiltl, struct aufilt *af);
 void aufilt_unregister(struct aufilt *af);
+void aufilt_enable(struct list *aufiltl, const char *name, bool enable);
 
 
 /*
@@ -723,6 +756,8 @@ typedef int  (menc_media_h)(struct menc_media **mp, struct menc_sess *sess,
 			   struct sdp_media *sdpm,
 			   const struct stream *strm);
 
+typedef int (menc_txrekey_h)(struct menc_media *m);
+
 struct menc {
 	struct le le;
 	const char *id;
@@ -730,6 +765,7 @@ struct menc {
 	bool wait_secure;
 	menc_sess_h *sessh;
 	menc_media_h *mediah;
+	menc_txrekey_h *txrekeyh;
 };
 
 void menc_register(struct list *mencl, struct menc *menc);
@@ -769,6 +805,7 @@ const struct sa *net_laddr_for(const struct network *net,
 bool net_is_laddr(const struct network *net, struct sa *sa);
 int net_set_dst_scopeid(const struct network *net, struct sa *dst);
 struct dnsc     *net_dnsc(const struct network *net);
+int net_set_dnsc(struct network *net, struct dnsc *dnsc);
 
 
 /*
@@ -830,13 +867,21 @@ enum ua_event {
 	UA_EVENT_AUDIO_ERROR,
 	UA_EVENT_CALL_LOCAL_SDP,      /**< param: offer or answer */
 	UA_EVENT_CALL_REMOTE_SDP,     /**< param: offer or answer */
+	UA_EVENT_CALL_HOLD,           /**< Call put on-hold by peer          */
+	UA_EVENT_CALL_RESUME,         /**< Call resumed by peer              */
 	UA_EVENT_REFER,
 	UA_EVENT_MODULE,
+	UA_EVENT_END_OF_FILE,
 	UA_EVENT_CUSTOM,
+	UA_EVENT_SIPSESS_CONN,
 
 	UA_EVENT_MAX,
 };
 
+
+struct bevent;
+
+
 /** SIP auto answer method */
 enum answer_method {
 	ANSM_NONE = 0,
@@ -848,6 +893,7 @@ enum answer_method {
 /** Defines the User-Agent event handler */
 typedef void (ua_event_h)(struct ua *ua, enum ua_event ev,
 			  struct call *call, const char *prm, void *arg);
+typedef void (bevent_h)(enum ua_event ev, struct bevent *event, void *arg);
 typedef void (options_resp_h)(int err, const struct sip_msg *msg, void *arg);
 typedef void (refer_resp_h)(int err, const struct sip_msg *msg, void *arg);
 
@@ -909,6 +955,8 @@ int ua_raise(struct ua *ua);
 int ua_set_autoanswer_value(struct ua *ua, const char *value);
 void ua_add_extension(struct ua *ua, const char *extension);
 void ua_remove_extension(struct ua *ua, const char *extension);
+bool ua_req_allowed(const struct ua *ua, const struct sip_msg *msg);
+bool ua_req_check_origin(const struct ua *ua, const struct sip_msg *msg);
 
 
 /* One instance */
@@ -1204,6 +1252,13 @@ struct videnc_state;
 struct viddec_state;
 struct vidcodec;
 
+struct viddec_packet {
+	bool intra;		      /**< True=keyframe, False=deltaframe   */
+	const struct rtp_header *hdr; /**< RTP Header                        */
+	uint64_t timestamp;	      /**< Timestamp in VIDEO_TIMEBASE units */
+	struct mbuf *mb;	      /**< RTP Buffer memory                 */
+};
+
 typedef int (videnc_packet_h)(bool marker, uint64_t rtp_ts,
 			      const uint8_t *hdr, size_t hdr_len,
 			      const uint8_t *pld, size_t pld_len,
@@ -1226,8 +1281,7 @@ typedef int(viddec_update_h)(struct viddec_state **vdsp,
 			     const struct video *vid);
 
 typedef int (viddec_decode_h)(struct viddec_state *vds, struct vidframe *frame,
-                              bool *intra, bool marker, uint16_t seq,
-                              struct mbuf *mb);
+                              struct viddec_packet *pkt);
 
 struct vidcodec {
 	struct le le;
@@ -1346,6 +1400,7 @@ struct stream *audio_strm(const struct audio *au);
 uint64_t audio_jb_current_value(const struct audio *au);
 int  audio_set_bitrate(struct audio *au, uint32_t bitrate);
 bool audio_rxaubuf_started(const struct audio *au);
+int  audio_update(struct audio *a);
 int  audio_start(struct audio *a);
 int  audio_start_source(struct audio *a, struct list *ausrcl,
 			struct list *aufiltl);
@@ -1361,6 +1416,7 @@ int  audio_decoder_set(struct audio *a, const struct aucodec *ac,
 const struct aucodec *audio_codec(const struct audio *au, bool tx);
 struct config_audio *audio_config(struct audio *au);
 bool audio_txtelev_empty(const struct audio *au);
+void audio_call_telev_handler(const struct audio *au, int key, bool end);
 
 
 /*
@@ -1374,6 +1430,7 @@ typedef void (video_err_h)(int err, const char *str, void *arg);
 int  video_alloc(struct video **vp, struct list *streaml,
 		 const struct stream_param *stream_prm,
 		 const struct config *cfg,
+		 const struct account *acc,
 		 struct sdp_session *sdp_sess,
 		 const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		 const struct menc *menc, struct menc_sess *menc_sess,
@@ -1397,6 +1454,7 @@ int   video_debug(struct re_printf *pf, const struct video *v);
 struct stream *video_strm(const struct video *v);
 const struct vidcodec *video_codec(const struct video *vid, bool tx);
 void video_sdp_attr_decode(struct video *v);
+void video_req_keyframe(struct video *vid);
 
 double video_calc_seconds(uint64_t rtp_ts);
 double video_timestamp_to_seconds(uint64_t timestamp);
@@ -1417,6 +1475,8 @@ struct stream_param {
 	const char *peer;   /**< Peer uri/name or identifier  */
 };
 
+struct jbuf_stat;
+
 typedef void (stream_mnatconn_h)(struct stream *strm, void *arg);
 typedef void (stream_rtpestab_h)(struct stream *strm, void *arg);
 typedef void (stream_rtcp_h)(struct stream *strm,
@@ -1425,6 +1485,7 @@ typedef void (stream_error_h)(struct stream *strm, int err, void *arg);
 
 int stream_update(struct stream *s);
 const struct rtcp_stats *stream_rtcp_stats(const struct stream *strm);
+int stream_jbuf_stats(const struct stream *strm, struct jbuf_stat *s);
 struct sdp_media *stream_sdpmedia(const struct stream *s);
 uint32_t stream_metric_get_tx_n_packets(const struct stream *strm);
 uint32_t stream_metric_get_tx_n_bytes(const struct stream *strm);
@@ -1442,6 +1503,7 @@ int  stream_start_mediaenc(struct stream *strm);
 int  stream_start_rtcp(const struct stream *strm);
 int  stream_enable(struct stream *strm, bool enable);
 int  stream_enable_tx(struct stream *strm, bool enable);
+int  stream_enable_rx(struct stream *strm, bool enable);
 void stream_mnat_attr(struct stream *strm, const char *name,
 		      const char *value);
 void stream_set_session_handlers(struct stream *strm,
@@ -1606,14 +1668,31 @@ void module_app_unload(void);
 
 int event_encode_dict(struct odict *od, struct ua *ua, enum ua_event ev,
 		      struct call *call, const char *prm);
+int odict_encode_bevent(struct odict *od, struct bevent *event);
 int event_add_au_jb_stat(struct odict *od_parent, const struct call *call);
 int  uag_event_register(ua_event_h *eh, void *arg);
 void uag_event_unregister(ua_event_h *eh);
+int  bevent_register(bevent_h *eh, void *arg);
+void bevent_unregister(bevent_h *eh);
 void ua_event(struct ua *ua, enum ua_event ev, struct call *call,
 	      const char *fmt, ...);
+int bevent_app_emit(enum ua_event ev, void *arg, const char *fmt, ...);
+int bevent_ua_emit(enum ua_event ev, struct ua *ua, const char *fmt, ...);
+int bevent_call_emit(enum ua_event ev, struct call *call,
+		     const char *fmt, ...);
+int bevent_sip_msg_emit(enum ua_event ev, const struct sip_msg *msg,
+			const char *fmt, ...);
 void module_event(const char *module, const char *event, struct ua *ua,
 		struct call *call, const char *fmt, ...);
 const char  *uag_event_str(enum ua_event ev);
+struct call    *bevent_get_call(const struct bevent *event);
+struct ua      *bevent_get_ua(const struct bevent *event);
+const struct sip_msg *bevent_get_msg(const struct bevent *event);
+void *bevent_get_apparg(const struct bevent *event);
+enum ua_event bevent_get_type(const struct bevent *event);
+const char *bevent_get_text(const struct bevent *event);
+void bevent_set_error(struct bevent *event, int err);
+void bevent_stop(struct bevent *event);
 
 
 /*
